<?php

namespace app\admin\logic;

use app\model\AccountModel;
use app\model\CommonModel;
use app\model\GoodModel;
use app\model\GroupModel;
use app\model\InventoryExchangeModel;
use think\Exception;
use think\exception\ValidateException;
use think\facade\Db;
use think\response\Json;

class InventoryExchangeLogic extends BaseLogic
{

    //获取兑换商品库存列表
    public static function list(array $data = []): Json
    {
        $db = InventoryExchangeModel::alias('a')
            ->leftJoin('account b', 'b.id=a.account_id AND b.is_del=' . CommonModel::$del_normal);

        if ($data['status'] != '') $db->where('b.status', $data['status']);
        if (($data['inventory_start'] != '') && ($data['inventory_end'] != '')) $db->whereBetween('a.inventory', [$data['inventory_start'], $data['inventory_end']]);
        if ($data['username'] != '') $db->whereLike('a.account_username', '%' . $data['username'] . '%');
        if ($data['name'] != '') $db->whereLike('a.account_name', '%' . $data['name'] . '%');
        if ($data['mobile'] != '') $db->whereLike('b.mobile', '%' . $data['mobile'] . '%');

        $count = $db->count('a.id');

        $list = $db->field('a.id,a.account_username,b.status,a.account_name,b.mobile,a.inventory,a.updatetime')
            ->page($data['page'], $data['size'])
            ->order(['a.id' => 'desc'])
            ->select()
            ->toArray();

        return json_show(CommonModel::$success, '获取兑换商品库存列表成功', ['count' => $count, 'list' => $list]);
    }

    //添加兑换商品库存
    public static function add(array $data = []): Json
    {
        Db::startTrans();
        try {

            $group = GroupModel::field('id,company_id,card_id')
                ->where(['id' => $data['group_id'], 'is_del' => CommonModel::$del_normal])
                ->findOrEmpty()
                ->toArray();
            if (empty($group)) throw new Exception('该分组不存在');

            $good_list = GoodModel::where(['is_del' => CommonModel::$del_normal, 'type' => GoodModel::$type_exchange])
                ->whereIn('id', array_column($data['list'], 'good_id'))
                ->column('id,good_code,good_name', 'id');

            if (empty($good_list)) throw new Exception('该商品不存在');

            $account_list = AccountModel::field('id,username,name')
                ->where(['is_del' => CommonModel::$del_normal, 'company_id' => $group['company_id'], 'card_id' => $group['card_id']])
                ->select()
                ->toArray();

            $temp = InventoryExchangeModel::field('id,account_username,good_name')
                ->whereIn('account_id', array_column($account_list, 'id'))
                ->whereIn('good_id', array_column($good_list, 'id'))
                ->findOrEmpty()
                ->toArray();
            if (!empty($temp)) throw new Exception('【' . $temp['account_username'] . '】账户的【' . $temp['good_name'] . '】商品库存记录已存在');

            $date = date('Y-m-d H:i:s');
            $inventory = array_column($data['list'], 'inventory', 'good_id');
            foreach ($account_list as $account) {
                foreach ($good_list as $good) {
                    $inventory_exchange_id = Db::name('inventory_exchange')->insertGetId([
                        'account_id' => $account['id'],
                        'account_username' => $account['username'],
                        'account_name' => $account['name'],
                        'good_id' => $good['id'],
                        'good_code' => $good['good_code'],
                        'good_name' => $good['good_name'],
                        'inventory' => $inventory[$good['id']] ?? 0,
                        'createrid' => self::$uid,
                        'creater' => self::$uname,
                        'addtime' => $date,
                        'updaterid' => self::$uid,
                        'updater' => self::$uname,
                        'updatetime' => $date,
                    ]);
                    Db::name('inventory_exchange_log')->insert([
                        'inventory_exchange_id' => $inventory_exchange_id,
                        'before_inventory' => 0,
                        'after_inventory' => $inventory[$good['id']] ?? 0,
                        'good_id' => $good['id'],
                        'good_code' => $good['good_code'],
                        'good_name' => $good['good_name'],
                        'source' => CommonModel::$source_admin,
                        'createrid' => self::$uid,
                        'creater' => self::$uname,
                        'addtime' => $date,
                    ]);
                }
            }

            Db::commit();

            return json_show(CommonModel::$success, '添加兑换商品库存成功');

        } catch (Exception $exception) {
            Db::rollback();
            return json_show(CommonModel::$error_param, '添加兑换商品库存失败，' . $exception->getMessage());
        }

    }

    //获取兑换商品库存详情
    public static function read(int $id = 0): Json
    {
        $res = InventoryExchangeModel::field(true)
            ->where(['id' => $id])
            ->findOrEmpty()
            ->toArray();

        if (empty($res)) throw new ValidateException('该兑换商品库存为空');

        return json_show(CommonModel::$success, '获取兑换商品库存详情成功', $res);
    }

    //编辑兑换商品库存
    public static function edit(array $data = []): Json
    {
        Db::startTrans();
        try {

            $rs = InventoryExchangeModel::field('id,inventory,good_id,good_code,good_name')
                ->lock(true)
                ->where('id', $data['id'])
                ->findOrEmpty();
            if ($rs->isEmpty()) throw new Exception('该兑换商品库存不存在');

            $date = date('Y-m-d H:i:s');

            if ($rs->inventory != $data['inventory']) {
                Db::name('inventory_exchange_log')->insert([
                    'inventory_exchange_id' => $data['id'],
                    'before_inventory' => $rs->inventory,
                    'after_inventory' => $data['inventory'],
                    'good_id' => $rs->good_id,
                    'good_code' => $rs->good_code,
                    'good_name' => $rs->good_name,
                    'source' => CommonModel::$source_admin,
                    'createrid' => self::$uid,
                    'creater' => self::$uname,
                    'addtime' => $date,
                ]);
            }

            InventoryExchangeModel::where('id', $data['id'])
                ->save(array_merge($data, ['updatetime' => $date, 'updaterid' => self::$uid, 'updater' => self::$uname]));

            Db::commit();

            return json_show(CommonModel::$success, '编辑兑换商品库存成功');

        } catch (Exception $exception) {
            Db::rollback();
            return json_show(CommonModel::$error_param, '编辑兑换商品库存失败，' . $exception->getMessage());
        }

    }

    //兑换商品库存变动记录
    public static function log(array $data = []): Json
    {
        $count = Db::name('inventory_exchange_log')
            ->where('inventory_exchange_id', $data['id'])
            ->count('id');

        $res = Db::name('inventory_exchange_log')
            ->field('id,before_inventory,after_inventory,good_id,good_code,good_name,source,creater,addtime')
            ->where('inventory_exchange_id', $data['id'])
            ->page($data['page'], $data['size'])
            ->order('id', 'desc')
            ->select()
            ->toArray();

        return $res ? json_show(CommonModel::$success, '获取兑换商品库存变动记录成功', ['count' => $count, 'list' => $res]) : json_show(CommonModel::$success, '获取兑换商品库存变动记录失败');

    }


}